@@ -636,10 +636,6 @@ static int generic_menu_cb(struct fdisk_context **cxt0,
 			break;
 		case 'f':
 			rc = fdisk_reorder_partitions(cxt);
-			if (rc)
-				fdisk_warnx(cxt, _("Failed to fix partitions order."));
-			else
-				fdisk_info(cxt, _("Partitions order fixed."));
 			break;
 		case 'r':
 			rc = fdisk_enable_details(cxt, 0);
