@@ -14,6 +14,7 @@
 #endif
 #include <fcntl.h>
 #include <unistd.h>
+#include <sys/stat.h>
 
 #ifdef HAVE_SYS_MKDEV_H
 # include <sys/mkdev.h>		/* major and minor on Solaris */
@@ -97,6 +98,9 @@ struct hd_geometry {
 /* are we working with block device? */
 int is_blkdev(int fd);
 
+/* open block device or file */
+int open_blkdev_or_file(const struct stat *st, const char *name, const int oflag);
+
 /* Determine size in bytes */
 off_t blkdev_find_size (int fd);
 
@@ -142,5 +146,6 @@ int blkdev_get_geometry(int fd, unsigned int *h, unsigned int *s);
 /* convert scsi type code to name */
 const char *blkdev_scsi_type_to_name(int type);
 
+int blkdev_lock(int fd, const char *devname, const char *lockmode);
 
 #endif /* BLKDEV_H */
