@@ -342,10 +342,6 @@ static inline int drop_permissions(void)
 {
 	errno = 0;
 
-	/* drop supplementary groups */
-	if (setgroups(0, NULL) != 0)
-		goto fail;
-
 	/* drop GID */
 	if (setgid(getgid()) < 0)
 		goto fail;
