@@ -36,7 +36,7 @@
 #include "debug.h"
 
 /*
- * Default behavior, maybe be override by terminal-colors.d/{enable,disable}.
+ * Default behavior, may be overridden by terminal-colors.d/{enable,disable}.
  */
 #ifdef USE_COLORS_BY_DEFAULT
 # define UL_COLORMODE_DEFAULT	UL_COLORMODE_AUTO	/* check isatty() */
@@ -424,8 +424,10 @@ static int cn_sequence(const char *str, char **seq)
 		in++;
 	}
 
-	assert ((out - *seq) <= len);
-	*out = '\0';
+	if (out) {
+		assert ((out - *seq) <= len);
+		*out = '\0';
+	}
 
 	return 0;
 }
@@ -653,26 +655,20 @@ static int colors_terminal_is_ready(void)
 {
 	int ncolors = -1;
 
-	if (isatty(STDOUT_FILENO) != 1)
-		goto none;
-
 #if defined(HAVE_LIBNCURSES) || defined(HAVE_LIBNCURSESW)
 	{
 		int ret;
 
-		if (setupterm(NULL, STDOUT_FILENO, &ret) != 0 || ret != 1)
-			goto none;
-		ncolors = tigetnum("colors");
-		if (ncolors <= 2)
-			goto none;
+		if (setupterm(NULL, STDOUT_FILENO, &ret) == 0 && ret == 1)
+			ncolors = tigetnum("colors");
 	}
 #endif
-	if (ncolors != -1) {
+	if (1 < ncolors) {
 		DBG(CONF, ul_debug("terminal is ready (supports %d colors)", ncolors));
 		return 1;
 	}
-none:
-	DBG(CONF, ul_debug("terminal is NOT ready"));
+
+	DBG(CONF, ul_debug("terminal is NOT ready (no colors)"));
 	return 0;
 }
 
@@ -692,11 +688,16 @@ int colors_init(int mode, const char *name)
 	struct ul_color_ctl *cc = &ul_colors;
 
 	cc->utilname = name;
-	cc->mode = mode;
 
 	termcolors_init_debug();
 
-	if (mode == UL_COLORMODE_UNDEF && (ready = colors_terminal_is_ready())) {
+	if (mode != UL_COLORMODE_ALWAYS && !isatty(STDOUT_FILENO))
+		cc->mode = UL_COLORMODE_NEVER;
+	else
+		cc->mode = mode;
+
+	if (cc->mode == UL_COLORMODE_UNDEF
+	    && (ready = colors_terminal_is_ready())) {
 		int rc = colors_read_configuration(cc);
 		if (rc)
 			cc->mode = UL_COLORMODE_DEFAULT;
@@ -754,6 +755,14 @@ int colors_wanted(void)
 	return ul_colors.has_colors;
 }
 
+/*
+ * Returns mode
+ */
+int colors_mode(void)
+{
+	return ul_colors.mode;
+}
+
 /*
  * Enable @seq color
  */
