@@ -22,27 +22,25 @@ static int probe_aix_pt(blkid_probe pr,
 
 	if (blkid_partitions_need_typeonly(pr))
 		/* caller does not ask for details about partitions */
-		return 0;
+		return BLKID_PROBE_OK;
 
 	ls = blkid_probe_get_partlist(pr);
 	if (!ls)
-		goto err;
+		return BLKID_PROBE_NONE;
 
 	tab = blkid_partlist_new_parttable(ls, "aix", 0);
 	if (!tab)
-		goto err;
+		return -ENOMEM;
 
-	return 0;
-err:
-	return -1;
+	return BLKID_PROBE_OK;
 }
 
 /*
  * We know nothing about AIX on-disk structures. Everything what we know is the
  * magic number at begin of the disk.
  *
- * Note, Linux kernel is tring to be smart and AIX signature is ignored when
- * there is a valid DOS partitions table. We don't support such behaviour. All
+ * Note, Linux kernel is trying to be smart and AIX signature is ignored when
+ * there is a valid DOS partitions table. We don't support such behavior. All
  * fdisk-like programs has to properly wipe the fist sector. Everything other
  * is a bug.
  */
