@@ -46,7 +46,7 @@
  *
  * @PART_ENTRY_UUID: partition UUID (gpt, or pseudo IDs for MBR)
  *
- * @PART_ENTRY_TYPE: partition type, 0xNN (e.g 0x82) or type UUID (gpt only) or type string (mac)
+ * @PART_ENTRY_TYPE: partition type, 0xNN (e.g. 0x82) or type UUID (gpt only) or type string (mac)
  *
  * @PART_ENTRY_FLAGS: partition flags (e.g. boot_ind) or  attributes (e.g. gpt attributes)
  *
@@ -184,7 +184,7 @@ struct blkid_struct_partition {
 	unsigned long long flags;	/* partition flags / attributes */
 
 	int		partno;		/* partition number */
-	char		uuid[UUID_STR_LEN]; /* UUID (when supported by PT), e.g GPT */
+	char		uuid[UUID_STR_LEN]; /* UUID (when supported by PT), e.g. GPT */
 	unsigned char	name[128];	/* Partition in UTF8 name (when supported by PT), e.g. Mac */
 
 	blkid_parttable	tab;		/* partition table */
@@ -466,7 +466,7 @@ blkid_partition blkid_partlist_add_partition(blkid_partlist ls,
 	return par;
 }
 
-/* allows to modify used partitions numbers (for example for logical partitions) */
+/* can be used to modify used partitions numbers (for example for logical partitions) */
 int blkid_partlist_set_partno(blkid_partlist ls, int partno)
 {
 	if (!ls)
@@ -480,7 +480,7 @@ int blkid_partlist_increment_partno(blkid_partlist ls)
 	return ls ? ls->next_partno++ : -1;
 }
 
-/* allows to set "parent" for the next nested partition */
+/* can be used to set "parent" for the next nested partition */
 static int blkid_partlist_set_parent(blkid_partlist ls, blkid_partition par)
 {
 	if (!ls)
@@ -923,7 +923,7 @@ int blkid_partlist_numof_partitions(blkid_partlist ls)
  * blkid_partlist_get_table:
  * @ls: partitions list
  *
- * Returns: top-level partition table or NULL of there is not a partition table
+ * Returns: top-level partition table or NULL if there is not a partition table
  * on the device.
  */
 blkid_parttable blkid_partlist_get_table(blkid_partlist ls)
@@ -1040,8 +1040,9 @@ blkid_partition blkid_partlist_devno_to_partition(blkid_partlist ls, dev_t devno
 			if (prefix && strncasecmp(prefix, "part", 4) == 0) {
 				char *end = NULL;
 
+				errno = 0;
 				partno = strtol(prefix + 4, &end, 10);
-				if (prefix == end || (end && *end))
+				if (errno || prefix == end || (end && *end))
 					partno = 0;
 				else
 					rc = 0;		/* success */
@@ -1357,7 +1358,7 @@ int blkid_partition_set_utf8name(blkid_partition par, const unsigned char *name,
 	if (!par)
 		return -1;
 
-	blkid_encode_to_utf8(enc, par->name, sizeof(par->name), name, len);
+	ul_encode_to_utf8(enc, par->name, sizeof(par->name), name, len);
 	blkid_rtrim_whitespace(par->name);
 	return 0;
 }
@@ -1408,7 +1409,7 @@ const char *blkid_partition_get_uuid(blkid_partition par)
  * @par: partition
  *
  * Returns: proposed partition number (e.g. 'N' from sda'N') or -1 in case of
- * error. Note that the number is generate by library independently on your OS.
+ * error. Note that the number is generated by library independently of your OS.
  */
 int blkid_partition_get_partno(blkid_partition par)
 {
@@ -1443,9 +1444,9 @@ blkid_loff_t blkid_partition_get_start(blkid_partition par)
  * @par: partition
  *
  * WARNING: be very careful when you work with MS-DOS extended partitions. The
- *          library always returns full size of the partition. If you want add
- *          the partition to the Linux system (BLKPG_ADD_PARTITION ioctl) you
- *          need to reduce the size of the partition to 1 or 2 blocks. The
+ *          library always returns full size of the partition. If you want to
+ *          add the partition to the Linux system (BLKPG_ADD_PARTITION ioctl)
+ *          you need to reduce the size of the partition to 1 or 2 blocks. The
  *          rest of the partition has to be inaccessible for mkfs or mkswap
  *          programs, we need a small space for boot loaders only.
  *
@@ -1481,7 +1482,7 @@ int blkid_partition_set_type_string(blkid_partition par,
 	return 0;
 }
 
-/* Sets partition 'type' for PT where the type is defined by UUIDrather
+/* Sets partition 'type' for PT where the type is defined by UUID rather
  * than by number
  */
 int blkid_partition_set_type_uuid(blkid_partition par, const unsigned char *uuid)
@@ -1494,7 +1495,7 @@ int blkid_partition_set_type_uuid(blkid_partition par, const unsigned char *uuid
  * blkid_partition_get_type_string:
  * @par: partition
  *
- * The type string is supported by a small subset of partition tables (e.g Mac
+ * The type string is supported by a small subset of partition tables (e.g. Mac
  * and EFI GPT).  Note that GPT uses type UUID and this function returns this
  * UUID as string.
  *
