@@ -52,7 +52,7 @@ static int probe_sun_pt(blkid_probe pr,
 		goto err;
 
 	/* sectors per cylinder (partition offset is in cylinders...) */
-	spc = be16_to_cpu(l->nhead) * be16_to_cpu(l->nsect);
+	spc = (uint64_t) be16_to_cpu(l->nhead) * be16_to_cpu(l->nsect);
 
 	DBG(LOWPROBE, ul_debug("Sun VTOC sanity=%u version=%u nparts=%u",
 			be32_to_cpu(l->vtoc.sanity),
@@ -69,7 +69,7 @@ static int probe_sun_pt(blkid_probe pr,
 
 	/*
 	 * So that old Linux-Sun partitions continue to work,
-	 * alow the VTOC to be used under the additional condition ...
+	 * allow the VTOC to be used under the additional condition ...
 	 */
 	use_vtoc = use_vtoc || !(l->vtoc.sanity || l->vtoc.version || l->vtoc.nparts);
 
