@@ -180,6 +180,9 @@ struct dmesg_control {
 	ssize_t		kmsg_first_read;/* initial read() return code */
 	char		kmsg_buf[BUFSIZ];/* buffer to read kmsg data */
 
+	time_t		since;		/* filter records by time */
+	time_t		until;		/* filter records by time */
+
 	/*
 	 * For the --file option we mmap whole file. The unnecessary (already
 	 * printed) pages are always unmapped. The result is that we have in
@@ -303,6 +306,9 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_("     --time-format <format>  show timestamp using the given format:\n"
 		"                               [delta|reltime|ctime|notime|iso]\n"
 		"Suspending/resume will make ctime and iso timestamps inaccurate.\n"), out);
+	fputs(_("     --since <time>          display the lines since the specified time\n"), out);
+	fputs(_("     --until <time>          display the lines until the specified time\n"), out);
+
 	fputs(USAGE_SEPARATOR, out);
 	printf(USAGE_HELP_OPTIONS(29));
 	fputs(_("\nSupported log facilities:\n"), out);
@@ -552,7 +558,7 @@ static ssize_t read_syslog_buffer(struct dmesg_control *ctl, char **buf)
 	if (ctl->bufsize) {
 		sz = ctl->bufsize + 8;
 		*buf = xmalloc(sz * sizeof(char));
-		rc = klogctl(ctl->action, *buf, sz);
+		rc = klogctl(SYSLOG_ACTION_READ_ALL, *buf, sz);
 	} else {
 		sz = 16392;
 		while (1) {
@@ -566,9 +572,6 @@ static ssize_t read_syslog_buffer(struct dmesg_control *ctl, char **buf)
 			*buf = NULL;
 			sz *= 4;
 		}
-
-		if (rc > 0 && ctl->action == SYSLOG_ACTION_READ_CLEAR)
-			rc = klogctl(SYSLOG_ACTION_READ_CLEAR, *buf, sz);
 	}
 
 	return rc;
@@ -596,8 +599,6 @@ static ssize_t read_buffer(struct dmesg_control *ctl, char **buf)
 		 * Since kernel 3.5.0
 		 */
 		n = read_kmsg(ctl);
-		if (n == 0 && ctl->action == SYSLOG_ACTION_READ_CLEAR)
-			n = klogctl(SYSLOG_ACTION_CLEAR, NULL, 0);
 		break;
 	default:
 		abort();	/* impossible method -> drop core */
@@ -778,6 +779,11 @@ static int get_next_syslog_record(struct dmesg_control *ctl,
 	return 1;
 }
 
+static time_t record_time(struct dmesg_control *ctl, struct dmesg_record *rec)
+{
+	return ctl->boot_time.tv_sec + ctl->suspended_time + rec->tv.tv_sec;
+}
+
 static int accept_record(struct dmesg_control *ctl, struct dmesg_record *rec)
 {
 	if (ctl->fltr_lev && (rec->facility < 0 ||
@@ -788,6 +794,12 @@ static int accept_record(struct dmesg_control *ctl, struct dmesg_record *rec)
 			      !isset(ctl->facilities, rec->facility)))
 		return 0;
 
+	if (ctl->since && ctl->since >= record_time(ctl, rec))
+		return 0;
+
+	if (ctl->until && ctl->until <= record_time(ctl, rec))
+		return 0;
+
 	return 1;
 }
 
@@ -825,7 +837,7 @@ static struct tm *record_localtime(struct dmesg_control *ctl,
 				   struct dmesg_record *rec,
 				   struct tm *tm)
 {
-	time_t t = ctl->boot_time.tv_sec + ctl->suspended_time + rec->tv.tv_sec;
+	time_t t = record_time(ctl, rec);
 	return localtime_r(&t, tm);
 }
 
@@ -1073,10 +1085,10 @@ full_output:
 			mesg_size = strlen(line);
 			goto full_output;
 		}
-		free(mesg_copy);
 	}
 
 done:
+	free(mesg_copy);
 	putchar('\n');
 }
 
@@ -1333,7 +1345,9 @@ int main(int argc, char *argv[])
 	int colormode = UL_COLORMODE_UNDEF;
 	enum {
 		OPT_TIME_FORMAT = CHAR_MAX + 1,
-		OPT_NOESC
+		OPT_NOESC,
+		OPT_SINCE,
+		OPT_UNTIL
 	};
 
 	static const struct option longopts[] = {
@@ -1352,6 +1366,7 @@ int main(int argc, char *argv[])
 		{ "help",          no_argument,	      NULL, 'h' },
 		{ "kernel",        no_argument,       NULL, 'k' },
 		{ "level",         required_argument, NULL, 'l' },
+		{ "since",	   required_argument, NULL, OPT_SINCE },
 		{ "syslog",        no_argument,       NULL, 'S' },
 		{ "raw",           no_argument,       NULL, 'r' },
 		{ "read-clear",    no_argument,	      NULL, 'c' },
@@ -1361,6 +1376,7 @@ int main(int argc, char *argv[])
 		{ "noescape",      no_argument,       NULL, OPT_NOESC },
 		{ "notime",        no_argument,       NULL, 't' },
 		{ "nopager",       no_argument,       NULL, 'P' },
+		{ "until",         required_argument, NULL, OPT_UNTIL },
 		{ "userspace",     no_argument,       NULL, 'u' },
 		{ "version",       no_argument,	      NULL, 'V' },
 		{ "time-format",   required_argument, NULL, OPT_TIME_FORMAT },
@@ -1491,7 +1507,22 @@ int main(int argc, char *argv[])
 		case OPT_NOESC:
 			ctl.noesc = 1;
 			break;
-
+		case OPT_SINCE:
+		{
+			usec_t p;
+			if (parse_timestamp(optarg, &p) < 0)
+				errx(EXIT_FAILURE, _("invalid time value \"%s\""), optarg);
+			ctl.since = (time_t) (p / 1000000);
+			break;
+		}
+		case OPT_UNTIL:
+		{
+			usec_t p;
+			if (parse_timestamp(optarg, &p) < 0)
+				errx(EXIT_FAILURE, _("invalid time value \"%s\""), optarg);
+			ctl.until = (time_t) (p / 1000000);
+			break;
+		}
 		case 'h':
 			usage();
 		case 'V':
@@ -1561,12 +1592,18 @@ int main(int argc, char *argv[])
 			print_buffer(&ctl, buf, n);
 		if (!ctl.mmap_buff)
 			free(buf);
-		if (n < 0)
-			err(EXIT_FAILURE, _("read kernel buffer failed"));
 		if (ctl.kmsg >= 0)
 			close(ctl.kmsg);
-		break;
+		if (n < 0)
+			err(EXIT_FAILURE, _("read kernel buffer failed"));
+		else if (ctl.action == SYSLOG_ACTION_READ_CLEAR)
+			; /* fallthrough */
+		else
+			break;
 	case SYSLOG_ACTION_CLEAR:
+		if (klogctl(SYSLOG_ACTION_CLEAR, NULL, 0) < 0)
+			err(EXIT_FAILURE, _("clear kernel buffer failed"));
+		break;
 	case SYSLOG_ACTION_CONSOLE_OFF:
 	case SYSLOG_ACTION_CONSOLE_ON:
 		klog_rc = klogctl(ctl.action, NULL, 0);
