@@ -56,6 +56,38 @@ static const struct colinfo infos[] = {
 	[COL_NAME]  = {"NAME",  0.70, SCOLS_FL_TRUNC, N_("name"),        SCOLS_JSON_STRING},
 };
 
+/* make softirq friendly to end-user */
+struct softirq_desc {
+	char *irq;
+	char *desc;
+} softirq_descs[] = {
+	{ .irq = "HI", .desc = "high priority tasklet softirq" },
+	{ .irq = "TIMER", .desc = "timer softirq" },
+	{ .irq = "NET_TX", .desc = "network transmit softirq", },
+	{ .irq = "NET_RX", .desc = "network receive softirq" },
+	{ .irq = "BLOCK", .desc = "block device softirq" },
+	{ .irq = "IRQ_POLL", .desc = "IO poll softirq" },
+	{ .irq = "TASKLET", .desc = "normal priority tasklet softirq" },
+	{ .irq = "SCHED", .desc = "schedule softirq" },
+	{ .irq = "HRTIMER", .desc = "high resolution timer softirq" },
+	{ .irq = "RCU", .desc = "RCU softirq" },
+};
+
+static void get_softirq_desc(struct irq_info *curr)
+{
+	int i, size = ARRAY_SIZE(softirq_descs);
+
+	for (i = 0; i < size; i++) {
+		if (!strcmp(curr->irq, softirq_descs[i].irq))
+			break;
+	}
+
+	if (i < size)
+		curr->name = xstrdup(softirq_descs[i].desc);
+	else
+		curr->name = xstrdup("");
+}
+
 int irq_column_name_to_id(const char *name, size_t namesz)
 {
 	size_t i;
@@ -111,7 +143,7 @@ static struct libscols_table *new_scols_table(struct irq_output *out)
 	scols_table_enable_export(table, out->pairs);
 
 	if (out->json)
-		scols_table_set_name(table, _("interrupts"));
+		scols_table_set_name(table, "interrupts");
 
 	for (i = 0; i < out->ncolumns; i++) {
 		const struct colinfo *col = get_column_info(out, i);
@@ -133,18 +165,22 @@ static struct libscols_table *new_scols_table(struct irq_output *out)
 	return NULL;
 }
 
+static struct libscols_line *new_scols_line(struct libscols_table *table)
+{
+	struct libscols_line *line = scols_table_new_line(table, NULL);
+	if (!line) {
+		warn(_("failed to add line to output"));
+		return NULL;
+	}
+	return line;
+}
+
 static void add_scols_line(struct irq_output *out,
 			   struct irq_info *info,
 			   struct libscols_table *table)
 {
 	size_t i;
-	struct libscols_line *line;
-
-	line = scols_table_new_line(table, NULL);
-	if (!line) {
-		warn(_("failed to add line to output"));
-		return;
-	}
+	struct libscols_line *line = new_scols_line(table);
 
 	for (i = 0; i < out->ncolumns; i++) {
 		char *str = NULL;
@@ -195,7 +231,7 @@ static char *remove_repeated_spaces(char *str)
 /*
  * irqinfo - parse the system's interrupts
  */
-static struct irq_stat *get_irqinfo(void)
+static struct irq_stat *get_irqinfo(int softirq)
 {
 	FILE *irqfile;
 	char *line = NULL, *tmp;
@@ -209,7 +245,10 @@ static struct irq_stat *get_irqinfo(void)
 	stat->irq_info = xmalloc(sizeof(*stat->irq_info) * IRQ_INFO_LEN);
 	stat->nr_irq_info = IRQ_INFO_LEN;
 
-	irqfile = fopen(_PATH_PROC_INTERRUPTS, "r");
+	if (softirq)
+		irqfile = fopen(_PATH_PROC_SOFTIRQS, "r");
+	else
+		irqfile = fopen(_PATH_PROC_INTERRUPTS, "r");
 	if (!irqfile) {
 		warn(_("cannot open %s"), _PATH_PROC_INTERRUPTS);
 		goto free_stat;
@@ -227,10 +266,13 @@ static struct irq_stat *get_irqinfo(void)
 		stat->nr_active_cpu++;
 	}
 
+	stat->cpus =  xcalloc(stat->nr_active_cpu, sizeof(struct irq_cpu));
+
 	/* parse each line of _PATH_PROC_INTERRUPTS */
 	while (getline(&line, &len, irqfile) >= 0) {
 		unsigned long count;
-		int index, length;
+		size_t index;
+		int length;
 
 		tmp = strchr(line, ':');
 		if (!tmp)
@@ -246,21 +288,30 @@ static struct irq_stat *get_irqinfo(void)
 
 		tmp += 1;
 		for (index = 0; (index < stat->nr_active_cpu) && (tmp - line < length); index++) {
+			struct irq_cpu *cpu = &stat->cpus[index];
+
 			sscanf(tmp, " %10lu", &count);
 			curr->total += count;
+			cpu->total += count;
 			stat->total_irq += count;
+
 			tmp += 11;
 		}
 
-		if (tmp - line < length) {
-			/* strip all space before desc */
-			while (isspace(*tmp))
-				tmp++;
-			tmp = remove_repeated_spaces(tmp);
-			rtrim_whitespace((unsigned char *)tmp);
-			curr->name = xstrdup(tmp);
-		} else	/* no irq name string, we have to set '\0' here */
-			curr->name = xstrdup("");
+		/* softirq always has no desc, add additional desc for softirq */
+		if (softirq)
+			get_softirq_desc(curr);
+		else {
+			if (tmp - line < length) {
+				/* strip all space before desc */
+				while (isspace(*tmp))
+					tmp++;
+				tmp = remove_repeated_spaces(tmp);
+				rtrim_whitespace((unsigned char *)tmp);
+				curr->name = xstrdup(tmp);
+			} else /* no irq name string, we have to set '\0' here */
+				curr->name = xstrdup("");
+		}
 
 		if (stat->nr_irq == stat->nr_irq_info) {
 			stat->nr_irq_info *= 2;
@@ -276,6 +327,7 @@ static struct irq_stat *get_irqinfo(void)
 	fclose(irqfile);
  free_stat:
 	free(stat->irq_info);
+	free(stat->cpus);
 	free(stat);
 	free(line);
 	return NULL;
@@ -294,6 +346,7 @@ void free_irqstat(struct irq_stat *stat)
 	}
 
 	free(stat->irq_info);
+	free(stat->cpus);
 	free(stat);
 }
 
@@ -366,9 +419,91 @@ void set_sort_func_by_key(struct irq_output *out, char c)
 	}
 }
 
+struct libscols_table *get_scols_cpus_table(struct irq_output *out,
+					struct irq_stat *prev,
+					struct irq_stat *curr)
+{
+	struct libscols_table *table;
+	struct libscols_column *cl;
+	struct libscols_line *ln;
+	char colname[sizeof(stringify_value(LONG_MAX))];
+	size_t i;
+
+	if (prev) {
+		for (i = 0; i < curr->nr_active_cpu; i++) {
+			struct irq_cpu *pre = &prev->cpus[i];
+			struct irq_cpu *cur = &curr->cpus[i];
+
+			cur->delta = cur->total - pre->total;
+		}
+	}
+
+	table = scols_new_table();
+	if (!table) {
+		warn(_("failed to initialize output table"));
+		return NULL;
+	}
+	scols_table_enable_json(table, out->json);
+	scols_table_enable_noheadings(table, out->no_headings);
+	scols_table_enable_export(table, out->pairs);
+
+	if (out->json)
+		scols_table_set_name(table, _("cpu-interrupts"));
+	else
+		scols_table_new_column(table, "", 0, SCOLS_FL_RIGHT);
+
+	for (i = 0; i < curr->nr_active_cpu; i++) {
+		snprintf(colname, sizeof(colname), "cpu%zu", i);
+		cl = scols_table_new_column(table, colname, 0, SCOLS_FL_RIGHT);
+		if (cl == NULL) {
+			warnx(_("failed to initialize output column"));
+			goto err;
+		}
+		if (out->json)
+			scols_column_set_json_type(cl, SCOLS_JSON_STRING);
+	}
+
+	/* per cpu % of total */
+	ln = new_scols_line(table);
+	if (!ln || (!out->json && scols_line_set_data(ln, 0, "%irq:") != 0))
+		goto err;
+
+	for (i = 0; i < curr->nr_active_cpu; i++) {
+		struct irq_cpu *cpu = &curr->cpus[i];
+		char *str;
+
+		xasprintf(&str, "%0.1f", (double)((long double) cpu->total / (long double) curr->total_irq * 100.0));
+		if (str && scols_line_refer_data(ln, i + 1, str) != 0)
+			goto err;
+	}
+
+	/* per cpu % of delta */
+	ln = new_scols_line(table);
+	/* xgettext:no-c-format */
+	if (!ln || (!out->json && scols_line_set_data(ln, 0, _("%delta:")) != 0))
+		goto err;
+
+	for (i = 0; i < curr->nr_active_cpu; i++) {
+		struct irq_cpu *cpu = &curr->cpus[i];
+		char *str;
+
+		if (!curr->delta_irq)
+			continue;
+		xasprintf(&str, "%0.1f", (double)((long double) cpu->delta / (long double) curr->delta_irq * 100.0));
+		if (str && scols_line_refer_data(ln, i + 1, str) != 0)
+			goto err;
+	}
+
+	return table;
+ err:
+	scols_unref_table(table);
+	return NULL;
+}
+
 struct libscols_table *get_scols_table(struct irq_output *out,
 					      struct irq_stat *prev,
-					      struct irq_stat **xstat)
+					      struct irq_stat **xstat,
+					      int softirq)
 {
 	struct libscols_table *table;
 	struct irq_info *result;
@@ -377,7 +512,7 @@ struct libscols_table *get_scols_table(struct irq_output *out,
 	size_t i;
 
 	/* the stats */
-	stat = get_irqinfo();
+	stat = get_irqinfo(softirq);
 	if (!stat)
 		return NULL;
 
@@ -398,8 +533,11 @@ struct libscols_table *get_scols_table(struct irq_output *out,
 	sort_result(out, result, stat->nr_irq);
 
 	table = new_scols_table(out);
-	if (!table)
+	if (!table) {
+		free(result);
+		free_irqstat(stat);
 		return NULL;
+	}
 
 	for (i = 0; i < stat->nr_irq; i++)
 		add_scols_line(out, &result[i], table);
