@@ -26,6 +26,7 @@ top_srcdir=
 top_builddir=
 paraller_jobs=1
 has_asan_opt=
+has_ubsan_opt=
 
 function num_cpus()
 {
@@ -74,6 +75,10 @@ while [ -n "$1" ]; do
 		OPTS="$OPTS $1"
 		has_asan_opt="yes"
 		;;
+	--memcheck-ubsan)
+		OPTS="$OPTS $1"
+		has_ubsan_opt="yes"
+		;;
 	--use-system-commands)
 		OPTS="$OPTS $1"
 		SYSCOMMANDS="yes"
@@ -149,18 +154,34 @@ if [ -z "$top_srcdir" ]; then
 fi
 if [ -z "$top_builddir" ]; then
 	top_builddir="$TS_TOPDIR/.."
+	if [ -e "$top_builddir/build/meson.conf" ]; then
+		top_builddir="$TS_TOPDIR/../build"
+	fi
 fi
 
 OPTS="$OPTS --srcdir=$top_srcdir --builddir=$top_builddir"
 
 # Auto-enable ASAN to avoid conflicts between tests and binaries
 if [ -z "$has_asan_opt" ]; then
-	asan=$(awk '/^ASAN_LDFLAGS/ { print $3 }' $top_builddir/Makefile)
+        if [ -e "$top_builddir/Makefile" ]; then
+	    asan=$(awk '/^ASAN_LDFLAGS/ { print $3 }' $top_builddir/Makefile)
+        else
+            . "$top_builddir/meson.conf"
+        fi
 	if [ -n "$asan" ]; then
 		OPTS="$OPTS --memcheck-asan"
 	fi
 fi
 
+if [ -z "$has_ubsan_opt" ]; then
+	if [ -e "$top_builddir/Makefile" ]; then
+		ubsan=$(awk '/^UBSAN_LDFLAGS/ { print $3 }' $top_builddir/Makefile)
+	fi
+	if [ -n "$ubsan" ]; then
+		OPTS="$OPTS --memcheck-ubsan"
+	fi
+fi
+
 declare -a comps
 if [ -n "$SUBTESTS" ]; then
 	# selected tests only
@@ -174,7 +195,7 @@ if [ -n "$SUBTESTS" ]; then
 	done
 else
 	if [ -z "$SYSCOMMANDS" -a ! -f "$top_builddir/test_ttyutils" ]; then
-		echo "Tests not compiled! Run 'make check' to fix the problem."
+		echo "Tests not compiled! Run 'make check-programs' to fix the problem."
 		exit 1
 	fi
 
@@ -224,6 +245,7 @@ if [ "$paraller_jobs" -ne 1 ]; then
 fi
 
 count=0
+mkdir -p $top_builddir/tests/
 >| $top_builddir/tests/failures
 printf "%s\n" ${comps[*]} |
 	sort |
